/*
filename : MAX6675thermometer.c
date : 01-dec-2016
author : fhn
mcu : ATmega16A without crystal
*/
//==============================================================

#define F_CPU 1000000L 

#define TC_OPEN_ERR  	 -5000
#define TC_DEVICE_ID_ERR -6000

#define P__1		PORTB
#define P__2		PORTD
#define P__3		PORTC

#define P__4		PORTA
#define P__5		PINA
#define B__SO		0b00000001
#define B__CS		0b00000010
#define B__CK		0b00000100

#define B__MS	  0b10000000

//==============================================================

#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>

int TCAVAL=800;
int TCADJ = 0;
//==============================================================
void initPorts ( )
{
	DDRA = 0b00001110;
	DDRB = 0xFF;
	DDRC = 0xFF;
	DDRD = 0xFF;
}
//==============================================================
int getsscod ( int ascii_code )
{
	
	int code = 0;
		switch ( ascii_code )
		{							
			case 0:	code = 0b01000000;	break;
			case 1:	code = 0b01111001;	break;
			case 2:	code = 0b00100100;	break;
			case 3:	code = 0b00110000;	break;
			case 4:	code = 0b00011001;	break;
			case 5:	code = 0b00010010;	break;
			case 6:	code = 0b00000010;	break;
			case 7:	code = 0b01111000;	break;
			case 8:	code = 0b00000000;	break;
			case 9:	code = 0b00010000;	break;
			case '-': 	code = 0b00111111;	break;
			case ' ':  	code = 0b01111111;	break;
			case 'E':  	code = 0b00000110;	break;
			case 'P':	code = 0b00001100;	break;
			case 'O':	code = 0b01000000;	break;
			case 'r':  	code = 0b00101111;	break;
			default : break;
		} // end of switch

	return (code);

}
//==============================================================
void shostr ( char* str )
{
	P__3 = getsscod(str[0]);
	P__2 = getsscod(str[1]);
	P__1 = getsscod(str[2]);
	return;
}
//==============================================================
void shoval ( int val )
{
	int v = val;
	
	if ( val==TC_OPEN_ERR )
	{
	
		P__3 = getsscod( 'O' );
		P__2 = getsscod( 'P' );
		P__1 = getsscod( 'E' );
		return;
	}	
	
	else if ( val==TC_DEVICE_ID_ERR )
	{
		
		P__3 = getsscod( 'E' );
		P__2 = getsscod( 'r' );
		P__1 = getsscod( 'r' );
		return;
	}	
	else if ( val<-100 )
	{
		P__3 = getsscod( ' ' );
		P__2 = getsscod( ' ' );
		P__1 = getsscod( ' ' );
		return;
	}	

	
	if (v>999)
		P__3 |= B__MS;
	else
		P__3 &= (~B__MS);
	if ( v<0 )
	{
		v= v*-1;
		P__3 = getsscod('-');
	}
	else
	{

		v = v % 1000;
		P__3 = getsscod(v/100);

	}
	v = v % 100;
	P__2  = getsscod(v/10);
	P__1   = getsscod(v%10);
}
//==============================================================
int readTemperature ( )
{
	long serial_data = 0;
	int temp = 0;

	P__4 &= (~B__CK);
	_delay_us ( 5 );
	P__4 &= (~B__CS);  
	_delay_us ( 5 );

	for ( int i=15; i>=0; i-- )
	{
		P__4 &= (~B__CK);  
		_delay_us ( 5 );
		P__4 |= (B__CK);	  
		_delay_us ( 5 );

		if ( P__5 & B__SO )   
			serial_data += 1<<i;
		
	}
	P__4 &= (~B__CK);
	P__4 |= (B__CS);  

	if ( ( serial_data & 0x0002 ) != 0 ) 
	{
		temp = TC_DEVICE_ID_ERR;
	}	
	else if ( ( serial_data & 0x0004 ) != 0 ) 
	{
		temp = TC_OPEN_ERR;
	}
	else
	{
		temp = ((serial_data>>3) & 0x0FFF);
	}


	return temp; 
}
//==============================================================
void editalm()
{
	int loop=0;
	shoval ( TCAVAL );
	while (loop<40)
	{
		loop++;
		shoval ( TCAVAL );
		_delay_ms(150);
		shostr ( "   " );
		_delay_ms(50);
		if ((P__5 & 0b00100000 )==0)
		{
			if ( TCAVAL < 980 )
				TCAVAL = TCAVAL+10;
			shoval ( TCAVAL );
			_delay_ms(300);
			loop=0;
		}
		else if ((P__5 & 0b00010000 )==0)
		{
			if ( TCAVAL > 50 )
				TCAVAL = TCAVAL-10;
			shoval ( TCAVAL );
			_delay_ms(300);
			loop=0;
		}

	}
	eeprom_update_word ((void *) 10, TCAVAL ); 
}
//==============================================================
void editadj()
{
	int loop=0;
	shoval ( TCADJ );
	while (loop<40)
	{
		loop++;
		shoval ( TCADJ  );
		_delay_ms(150);
		shostr ( "   " );
		_delay_ms(50);
		if ((P__5 & 0b00100000 )==0)
		{
			if ( TCADJ < 25 )
				TCADJ = TCADJ+1;
			shoval (TCADJ );
			_delay_ms(300);
			loop=0;
		}
		else if ((P__5 & 0b00010000 )==0)
		{
			if ( TCADJ > -25 )
				TCADJ = TCADJ-1;
			shoval ( TCADJ );
			_delay_ms(300);
			loop=0;
		}

	}
	eeprom_update_byte ((void *) 1, 100+TCADJ ); 
}
//==============================================================
int main ( )
{

	int temperature = 0;
	int loop = -1;

	_delay_ms ( 100 );
	
	MCUCSR|=0x80;
	MCUCSR|=0x80;
	
	initPorts ( );
	_delay_ms ( 60 );

	shoval ( 1888 ); 
	_delay_ms ( 1000 );

	shostr ( "   " ); // blank
	_delay_ms ( 500 );


	TCAVAL = (int) eeprom_read_word((void *)10); 

	if ( TCAVAL > 2000 || TCAVAL < 0 )
	{
		TCAVAL = 800;
		eeprom_write_word ((void *) 10, 800); 
	}
	while (( TCAVAL % 10 ) != 0)
		TCAVAL++;
	

	shoval ( TCAVAL );
	_delay_ms ( 1500 );
	shostr ( "   " ); 
	_delay_ms ( 500 );


	TCADJ = (int) eeprom_read_byte((void *)1); 


	if (TCADJ > 200 || TCADJ < 0 )
	{
		TCADJ = 0;
		eeprom_write_byte ((void *) 1, 100); 
	}
	else
	{
		TCADJ -= 100;		
	}

	shoval ( TCADJ );
	_delay_ms ( 1000 );
	shostr ( "   " ); 
	_delay_ms ( 100 );


	while ( 1 )
	{
		loop++;
		if (loop>=1000)
			loop = 0;

		if ( (P__5 & 0b10000000 ) == 0)
		{
			P__1 &= (~B__MS);
			editalm();
			loop=0;
		}
		if ( (P__5 & 0b01000000 ) == 0)
		{
			P__1 &= (~B__MS);
			editadj();
			loop=0;
		}
		
		P__1 |= B__MS;
		_delay_ms ( 300 );
		P__2 &= (~B__MS); 
		P__1 &= (~B__MS);
		_delay_ms ( 300 );

		

		if (loop%4)
			continue;

		temperature = readTemperature ( );

		if ( temperature < -1 )
		{
			shoval ( temperature );
			temperature = 1999;
		}
		else
		{
			temperature /= 4;
			temperature += TCADJ;
			shoval ( temperature );
		}

		
		if (temperature>TCAVAL)
		{
			P__2 |= B__MS;  
			PORTA |= 0b00001000; 
		}
		else
			PORTA &= 0b11110111;  

	} 
	return 0;
}
//==============================================================


